from .unit import Unit, Skill
from .unittype import FlyingType
from .terrain import ParkingSlot
from .planes import PlaneType, A_10C
from .helicopters import HelicopterType, Ka_50

import json
from typing import Type


class FlyingUnit(Unit):
    def __init__(self, _id=None, name=None, _type: Type[FlyingType] = None, _country=None):
        super(FlyingUnit, self).__init__(_id, name, _type.id)
        self.unit_type = _type  # for loadout validation
        self.unit_type.load_payloads()
        self.livery_id = self.unit_type.default_livery(_country.name)
        self.parking = None  # crossroad idx
        self.parking_id = None  # parking slot name (01, 02, ..)
        self.psi = 0
        self.onboard_num = "010"
        self.alt = 0
        self.alt_type = "BARO"
        self.flare = _type.flare
        self.chaff = _type.chaff
        self.fuel = _type.fuel_max
        self.gun = 100
        self.ammo_type = _type.ammo_type
        self.pylons = {}
        self.callsign = None
        self.callsign_dict = {1: 1, 2: 1, 3: 1, "name": ""}
        self.speed = 0
        self.radio = None
        self.hardpoint_racks = True
        self.addpropaircraft = _type.property_defaults if _type.property_defaults else None

    def load_from_dict(self, d):
        super(FlyingUnit, self).load_from_dict(d)
        self.livery_id = d.get("livery_id")
        self.alt_type = d["alt_type"]
        self.alt = d["alt"]
        self.psi = d["psi"]
        self.speed = d["speed"]
        self.fuel = d["payload"]["fuel"]
        self.gun = d["payload"]["gun"]
        self.flare = d["payload"]["flare"]
        self.chaff = d["payload"]["chaff"]
        self.ammo_type = d["payload"].get("ammo_type")
        self.pylons = d["payload"]["pylons"]
        self.onboard_num = d["onboard_num"]
        if isinstance(d["callsign"], int):
            self.callsign = d["callsign"]
        else:
            self.callsign_dict = d["callsign"]
        self.parking = d.get("parking", None)
        self.parking_id = d.get("parking_id", None)
        if self.parking:
            self.parking = int(self.parking)
        self.radio = d.get("Radio")
        self.hardpoint_racks = d.get("hardpoint_racks", None)
        self.addpropaircraft = d.get("AddPropAircraft")
        return True

    def set_parking(self, parking_slot: ParkingSlot):
        parking_slot.unit_id = self.id
        self.parking = parking_slot.crossroad_idx
        self.parking_id = parking_slot.slot_name

    def set_property(self, prop_name, value):
        if self.addpropaircraft is None:
            self.addpropaircraft = {}
        self.addpropaircraft[prop_name] = value

    def load_pylon(self, weapon, pylon=None):
        if pylon is None:
            pylon = weapon[0]
        if pylon not in self.unit_type.pylons:
            raise RuntimeError("Plane {pn} has no pylon {p}.".format(pn=self.unit_type.id, p=pylon))
        if weapon is None:
            return self.pylons.pop(pylon, None)
        self.pylons[pylon] = {"CLSID": weapon[1]["clsid"]}
        return True

    def store_loadout(self, filename):
        with open(filename, 'w') as loadout:
            json.dump(self.pylons, loadout)

        return True

    def load_loadout(self, filename):
        with open(filename, 'r') as loadout:
            self.pylons = json.load(loadout)

        return True

    def reset_loadout(self):
        self.pylons = {}

    def set_default_preset_channel(self, freq):
        if self.radio:
            self.radio[1]["channels"][1] = freq

    def set_radio_preset(self):
        if self.unit_type.panel_radio:
            self.radio = self.unit_type.panel_radio

    def set_player(self):
        if not self.unit_type.flyable:
            raise RuntimeError("Unittype '{t}' is not human flyable".format(t=self.unit_type.id))
        self.skill = Skill.Player
        self.set_radio_preset()

    def set_client(self):
        if not self.unit_type.flyable:
            raise RuntimeError("Unittype '{t}' is not human flyable".format(t=self.unit_type.id))
        self.skill = Skill.Client
        self.set_radio_preset()

    def is_human(self):
        return self.skill in [Skill.Player, Skill.Client]

    def dict(self):
        d = super(FlyingUnit, self).dict()
        d["alt"] = self.alt
        d["alt_type"] = self.alt_type
        if self.parking is not None:
            d["parking"] = self.parking
        if self.parking_id is not None:
            d["parking_id"] = self.parking_id
        if self.livery_id:
            d["livery_id"] = self.livery_id
        d["psi"] = self.psi
        d["onboard_num"] = self.onboard_num
        d["speed"] = round(self.speed, 13)
        if self.hardpoint_racks is not None:
            d["hardpoint_racks"] = self.hardpoint_racks
        if self.addpropaircraft is not None:
            d["AddPropAircraft"] = self.addpropaircraft
        d["payload"] = {
            "flare": self.flare,
            "chaff": self.chaff,
            "fuel": self.fuel,
            "gun": self.gun,
            "pylons": self.pylons
        }
        if self.ammo_type:
            d["payload"]["ammo_type"] = self.ammo_type
        if self.callsign:
            d["callsign"] = self.callsign
        else:
            d["callsign"] = self.callsign_dict
        if self.radio:
            d["Radio"] = self.radio
        return d


class Plane(FlyingUnit):
    def __init__(self, _id=None, name=None, _type: Type[PlaneType] = A_10C, _country=None):
        super(Plane, self).__init__(_id, name, _type, _country)


class Helicopter(FlyingUnit):
    def __init__(self, _id=None, name=None, _type: Type[HelicopterType] = Ka_50, _country=None):
        super(Helicopter, self).__init__(_id, name, _type, _country)
        self.rope_length = 15

    def load_from_dict(self, d):
        super(Helicopter, self).load_from_dict(d)
        self.rope_length = d["ropeLength"]

    def dict(self):
        d = super(Helicopter, self).dict()
        d["ropeLength"] = self.rope_length
        return d


import unittest
from dcs.lua.parse import loads


class TestLuaParse(unittest.TestCase):

    def test_integer(self):
        r = loads("int = 3")
        self.assertEqual(r, {"int": 3})

        r = loads("int = 193984")
        self.assertEqual(r, {"int": 193984})

        r = loads("int = -23")
        self.assertEqual(r, {"int": -23})

    def test_decimal(self):
        r = loads("dec = 666.6")
        self.assertEqual(r, {"dec": 666.6})

    def test_exponent(self):
        r = loads("dec = 666.6e10")
        self.assertEqual(r, {"dec": 666.6e10})

        r = loads("dec = 666.6e-10")
        self.assertEqual(r, {"dec": 666.6e-10})

    def test_string(self):
        s = """
mission =
{
    ["trig"] =
    {
        ["actions"] =
        {
            [1] = "a_(get(\\"ResKey_Action_62\\")); mission.trig.func[1]=nil;",
            [2] = "a_do_script_file(getValueResourceByKey(\\"ResKey_Action_64\\")); mission.trig.func[2]=nil;",
            [3] = "a_out_text_delay(getValueDictByKey(\\"DictKey_ActionText_66\\"), 10, false);a_do_script(getValueDictByKey(\\"DictKey_ActionText_255\\")); mission.trig.func[3]=nil;",
            [4] = "a_add_radio_item(getValueDictByKey(\\"Destroy Red Ground Group\\"), 1500, 1); mission.trig.func[4]=nil;",
            [5] = "a_do_script(getValueDictByKey(\\"DictKey_ActionText_213\\"));a_clear_flag(1500);",
        }, -- end of ["actions"]
    }
}
"""
        r = loads(s)

        r = loads('x = "a_do_script_file(getValueResourceByKey(\\"ResKey_Action_62\\")); mission.trig.func[1]=nil;"')
        self.assertEqual(r, {
            "x": "a_do_script_file(getValueResourceByKey(\"ResKey_Action_62\")); mission.trig.func[1]=nil;"})

    def test_object(self):
        luas = """
mission=
{
    ["trig"]=
    {
    },

    ["coalitions"]=
    {
        ["blue"]=
        {
            [1]=11,
            [2]=4,
            [3]=6
        }
    }, -- end of ["coalitions"]
    ["maxDictId"]=18,
    ["descriptionBlueTask"]="DictKey_descriptionBlueTask_3"
}
"""
        ref = {'mission': {
            'coalitions': {'blue': {1: 11.0, 2: 4.0, 3: 6.0}},
            'descriptionBlueTask': 'DictKey_descriptionBlueTask_3',
            'trig': {},
            'maxDictId': 18.0}}
        r = loads(luas)
        self.assertEqual(r, ref)

    def test_dictmix(self):
        luas = """
o =
{
    ["callsign"] =
    {
        [1] = 1,
        [2] = 1,
        [3] = 1,
        ["name"] = "Enfield11",
    } -- end of ["callsign"]
}
"""
        ref = {"o": {
            'callsign': {
                1: 1,
                2: 1,
                3: 1,
                "name": "Enfield11"
            }
        }}
        r = loads(luas)
        self.assertEqual(r, ref)

    def test_syntaxerr(self):
        with self.assertRaises(SyntaxError):
            loads("""m=
            {
                ["x"] 12
            }""")

    def test_missing_curly(self):
        with self.assertRaises(SyntaxError):
            loads("""t=
            {
                ["payload"] = {
                    ["num"] = 1
            }
            """)

    def test_object_without_keys(self):
        luas = """
local unitPayloads = {
    ["name"] = "JF-17",
    ["payloads"] = {
        {
            ["name"] = "PL-5Ex2, C802AKx2, 800L Tank",
            ["pylons"] = {
                [1] = {
                    ["CLSID"] = "DIS_C-802AK",
                    ["num"] = 5
                }
            }
        }
    }
}
        """
        ref = {'unitPayloads': {
            'name': "JF-17",
            'payloads': {
                1: {
                    'name': "PL-5Ex2, C802AKx2, 800L Tank",
                    'pylons': {
                        1: {
                            'CLSID': "DIS_C-802AK",
                            'num': 5
                        }
                    }
                }
            }
        }}
        r = loads(luas)
        self.assertEqual(r, ref)

    def test_mixed_objects(self):
        luas = """
o = {
    "x",
    ["a"]=2,
    "y"
}
"""

        ref = {'o': {1: "x", "a": 2, 2: "y"}}
        r = loads(luas)
        self.assertEqual(r, ref)

    def test_payload_var_ref(self):
        luas = """
local pylon_1A,pylon_1B,pylon_2,pylon_3,pylon_4,pylon_5,pylon_6,pylon_7,pylon_8B,pylon_8A = 1,2,3,4,5,6,7,8,9,10

local unitPayloads = {
  ["name"] = "F-14B",
  ["payloads"] = {
    {
      ["name"] = "XT*2",
      ["pylons"] = {
        [1] = {
          ["CLSID"] = "{F14-300gal}", ["num"] = pylon_7,
        },
        [2] = {
          ["CLSID"] = "{F14-300gal}", ["num"] = pylon_2,
        },
      },
      ["tasks"] = {
        [1] = Intercept,
        [2] = CAP,
        [3] = Escort,
        [4] = FighterSweep,
      },
    },
    {
      ["name"] = "AIM-54A-MK47*6, AIM-9M*2, XT*2",
      ["pylons"] = {
        [1] = {
          ["CLSID"] = "{LAU-138 wtip - AIM-9M}", ["num"] = pylon_8A,
        },
        [2] = {
          ["CLSID"] = "{SHOULDER AIM_54A_Mk47 R}", ["num"] = pylon_8B,
        },
        [3] = {
          ["CLSID"] = "{F14-300gal}", ["num"] = pylon_7,
        },
        [4] = {
          ["CLSID"] = "{AIM_54A_Mk47}", ["num"] = pylon_6,
        },
        [5] = {
          ["CLSID"] = "{AIM_54A_Mk47}", ["num"] = pylon_5,
        },
        [6] = {
          ["CLSID"] = "{AIM_54A_Mk47}", ["num"] = pylon_4,
        },
        [7] = {
          ["CLSID"] = "{AIM_54A_Mk47}", ["num"] = pylon_3,
        },
        [8] = {
          ["CLSID"] = "{F14-300gal}", ["num"] = pylon_2,
        },
        [9] = {
          ["CLSID"] = "{SHOULDER AIM_54A_Mk47 L}", ["num"] = pylon_1B,
        },
        [10] = {
          ["CLSID"] = "{LAU-138 wtip - AIM-9M}", ["num"] = pylon_1A,
        },
      },
      ["tasks"] = {
        [1] = Intercept,
        [2] = CAP,
        [3] = Escort,
        [4] = FighterSweep,
      },
    },
  },
  ["unitType"] = "F-14B",
}
return unitPayloads
"""
        r = loads(luas, {'Intercept': 'Intercept', 'CAP': 'CAP', 'Escort': 'Escort', 'FighterSweep': 'FighterSweep'})
        self.assertEqual(r['unitPayloads']['name'], 'F-14B')
        self.assertEqual(r['unitPayloads']['payloads'][1]['name'], "XT*2")
        self.assertEqual(r['unitPayloads']['payloads'][1]['pylons'][1]['num'], 8)
        self.assertEqual(r['unitPayloads']['payloads'][2]['name'], "AIM-54A-MK47*6, AIM-9M*2, XT*2")
        self.assertEqual(r['unitPayloads']['payloads'][2]['tasks'][1], "Intercept")


if __name__ == '__main__':
    unittest.main()



from dcs import mapping
from dcs.terrain import Airport, Runway, ParkingSlot, Terrain, MapView


class Abbeville_Drucat(Airport):
    id = 1
    name = "Abbeville Drucat"
    position = mapping.Point(-81676.550781, 15906.308105)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Abbeville_Drucat, self).__init__()

        self.runways.append(Runway(20))
        self.runways.append(Runway(90))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(-80735.7109375, 16925.8671875), large=False, heli=True,
                airplanes=True, slot_name='02', length=26.0, width=24.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(-81904.96875, 18212.265625), large=False, heli=True,
                airplanes=True, slot_name='27', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(-81981.75, 18186.1328125), large=False, heli=True,
                airplanes=True, slot_name='26', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(-82036.9921875, 18140.03125), large=False, heli=True,
                airplanes=True, slot_name='25', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(-82104.96875, 18073.689453125), large=False, heli=False,
                airplanes=True, slot_name='22', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(-82129.5390625, 18024.455078125), large=False, heli=False,
                airplanes=True, slot_name='21', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(-82251.1328125, 17997.052734375), large=False, heli=True,
                airplanes=True, slot_name='20', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(-82177.9140625, 18076.76953125), large=False, heli=True,
                airplanes=True, slot_name='23', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(-82156.6171875, 18166.736328125), large=False, heli=True,
                airplanes=True, slot_name='24', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(-80611.53125, 17394.251953125), large=False, heli=False,
                airplanes=True, slot_name='07', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(-80539.515625, 17308.9140625), large=False, heli=True,
                airplanes=True, slot_name='06', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(-80572.734375, 17265.576171875), large=False, heli=True,
                airplanes=True, slot_name='05', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(-80796.953125, 17540.17578125), large=False, heli=True,
                airplanes=True, slot_name='08', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(-80770.890625, 17580.666015625), large=False, heli=True,
                airplanes=True, slot_name='09', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(-80780.75, 17636.404296875), large=False, heli=True,
                airplanes=True, slot_name='10', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(-80849.515625, 17666.310546875), large=False, heli=True,
                airplanes=True, slot_name='11', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(-80876.828125, 17769.578125), large=False, heli=False,
                airplanes=True, slot_name='12', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(-80916.1171875, 17871.734375), large=False, heli=False,
                airplanes=True, slot_name='13', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(-80914.4921875, 17947.568359375), large=False, heli=False,
                airplanes=True, slot_name='14', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(-80921.890625, 18065.88671875), large=False, heli=False,
                airplanes=True, slot_name='15', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(-80939.0234375, 18224.556640625), large=False, heli=False,
                airplanes=True, slot_name='16', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(-80966.4453125, 18286.35546875), large=False, heli=True,
                airplanes=True, slot_name='17', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(-80920.09375, 18382.9296875), large=False, heli=True,
                airplanes=True, slot_name='18', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(-80843.6640625, 18428.873046875), large=False, heli=True,
                airplanes=True, slot_name='19', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(-80718.4921875, 17059.05859375), large=False, heli=True,
                airplanes=True, slot_name='03', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(-80717.234375, 17166.173828125), large=False, heli=True,
                airplanes=True, slot_name='04', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(-80798.8046875, 16778.943359375), large=False, heli=False,
                airplanes=True, slot_name='01', length=26.0, width=24.0, height=11.0, shelter=False))


class Merville_Calonne(Airport):
    id = 2
    name = "Merville Calonne"
    position = mapping.Point(-29173.647461, 74342.222656)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Merville_Calonne, self).__init__()

        self.runways.append(Runway(210))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(-28799.3125, 73304.28125), large=False, heli=True,
                airplanes=True, slot_name='01', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(-28877.72265625, 73330.0625), large=False, heli=True,
                airplanes=True, slot_name='02', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(-28947.390625, 73366.6484375), large=False, heli=True,
                airplanes=True, slot_name='03', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(-29033.779296875, 73389.8125), large=False, heli=True,
                airplanes=True, slot_name='04', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(-29095.7578125, 73439.578125), large=False, heli=True,
                airplanes=True, slot_name='05', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(-29123.04296875, 73529.109375), large=False, heli=True,
                airplanes=True, slot_name='06', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(-29171.232421875, 73592.3203125), large=False, heli=True,
                airplanes=True, slot_name='07', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(-29233.537109375, 73642.328125), large=False, heli=True,
                airplanes=True, slot_name='08', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(-29314.5, 73666.9609375), large=False, heli=True,
                airplanes=True, slot_name='09', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(-28822.259765625, 73455.8125), large=False, heli=True,
                airplanes=True, slot_name='10', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(-28874.470703125, 73517.0078125), large=False, heli=True,
                airplanes=True, slot_name='11', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(-28899.98828125, 73610.71875), large=False, heli=True,
                airplanes=True, slot_name='12', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(-28992.685546875, 73620.203125), large=False, heli=True,
                airplanes=True, slot_name='13', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(-29044.515625, 73681.5625), large=False, heli=True,
                airplanes=True, slot_name='14', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(-29080.21875, 73759.6484375), large=False, heli=True,
                airplanes=True, slot_name='15', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(-29189.1171875, 73756.984375), large=False, heli=True,
                airplanes=True, slot_name='16', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(-29730.138671875, 74204.2890625), large=False, heli=True,
                airplanes=True, slot_name='24', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(-29778.630859375, 74266.1640625), large=False, heli=True,
                airplanes=True, slot_name='25', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(-29878.7578125, 74328.1796875), large=False, heli=True,
                airplanes=True, slot_name='26', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(-29951.150390625, 74389.2421875), large=False, heli=True,
                airplanes=True, slot_name='27', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(-30050.033203125, 74320.78125), large=False, heli=True,
                airplanes=True, slot_name='28', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(-29824.75, 74133.171875), large=False, heli=True,
                airplanes=True, slot_name='23', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(-29924.77734375, 74115.6796875), large=False, heli=True,
                airplanes=True, slot_name='22', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(-29998.619140625, 74074.3515625), large=False, heli=True,
                airplanes=True, slot_name='21', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(-30025.546875, 73993.875), large=False, heli=True,
                airplanes=True, slot_name='20', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(-30101.873046875, 73954.5859375), large=False, heli=True,
                airplanes=True, slot_name='19', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(-30011.443359375, 74237.671875), large=False, heli=True,
                airplanes=True, slot_name='29', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(-30087.978515625, 74200.8671875), large=False, heli=True,
                airplanes=True, slot_name='30', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(-30127.7890625, 74129.8671875), large=False, heli=True,
                airplanes=True, slot_name='31', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=31, position=mapping.Point(-30191.65234375, 74081.359375), large=False, heli=True,
                airplanes=True, slot_name='32', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=32, position=mapping.Point(-30251.1640625, 73903.1875), large=False, heli=True,
                airplanes=True, slot_name='18', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=33, position=mapping.Point(-30272.64453125, 73826.453125), large=False, heli=True,
                airplanes=True, slot_name='17', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=34, position=mapping.Point(-28323.58203125, 75295.2578125), large=False, heli=True,
                airplanes=True, slot_name='42', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=35, position=mapping.Point(-28401.041015625, 75344.8046875), large=False, heli=True,
                airplanes=True, slot_name='41', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=36, position=mapping.Point(-28518.671875, 75338.5), large=False, heli=True,
                airplanes=True, slot_name='40', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=37, position=mapping.Point(-28634.095703125, 75361.375), large=False, heli=True,
                airplanes=True, slot_name='39', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=38, position=mapping.Point(-28711.80078125, 75333.7265625), large=False, heli=True,
                airplanes=True, slot_name='38', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=39, position=mapping.Point(-28450.36328125, 75186.1640625), large=False, heli=True,
                airplanes=True, slot_name='43', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=40, position=mapping.Point(-28527.072265625, 75173.59375), large=False, heli=True,
                airplanes=True, slot_name='44', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=41, position=mapping.Point(-28607.244140625, 75176.359375), large=False, heli=True,
                airplanes=True, slot_name='45', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=42, position=mapping.Point(-28683.728515625, 75216.828125), large=False, heli=True,
                airplanes=True, slot_name='46', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=43, position=mapping.Point(-28722.900390625, 75104.671875), large=False, heli=True,
                airplanes=True, slot_name='47', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=44, position=mapping.Point(-28696.640625, 74998.0390625), large=False, heli=True,
                airplanes=True, slot_name='48', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=45, position=mapping.Point(-28811.0859375, 74962.1796875), large=False, heli=True,
                airplanes=True, slot_name='33', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=46, position=mapping.Point(-28830.884765625, 75039.484375), large=False, heli=True,
                airplanes=True, slot_name='34', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=47, position=mapping.Point(-28847.97265625, 75124.5078125), large=False, heli=True,
                airplanes=True, slot_name='35', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=48, position=mapping.Point(-28833.287109375, 75208.5625), large=False, heli=True,
                airplanes=True, slot_name='36', length=40.0, width=40.0, height=12.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=49, position=mapping.Point(-28786.783203125, 75283.796875), large=False, heli=True,
                airplanes=True, slot_name='37', length=40.0, width=40.0, height=12.0, shelter=False))


class Saint_Omer_Longuenesse(Airport):
    id = 3
    name = "Saint Omer Longuenesse"
    position = mapping.Point(-16951.71582, 45171.712891)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Saint_Omer_Longuenesse, self).__init__()

        self.runways.append(Runway(260))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(-16750.32421875, 44658.95703125), large=False, heli=False,
                airplanes=True, slot_name='03', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(-16685.8984375, 44518.21484375), large=False, heli=False,
                airplanes=True, slot_name='01', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(-16855.115234375, 44527.1484375), large=False, heli=False,
                airplanes=True, slot_name='02', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(-16827.42990945, 44712.185374436), large=False, heli=False,
                airplanes=True, slot_name='04', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(-16854.392330572, 44729.218354763), large=False, heli=False,
                airplanes=True, slot_name='05', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(-16877.047453213, 44750.456918223), large=False, heli=False,
                airplanes=True, slot_name='06', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(-16722.392578125, 44772.96875), large=False, heli=False,
                airplanes=True, slot_name='10', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(-16689.876953125, 44806.828125), large=False, heli=False,
                airplanes=True, slot_name='11', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(-16652.298797382, 44860.903628482), large=False, heli=False,
                airplanes=True, slot_name='12', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(-16595.658713604, 44887.251680392), large=False, heli=False,
                airplanes=True, slot_name='15', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(-16553.65973541, 44920.352995763), large=False, heli=False,
                airplanes=True, slot_name='16', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(-16593.215956891, 44964.096865776), large=False, heli=False,
                airplanes=True, slot_name='17', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(-16627.873343939, 44935.366347516), large=False, heli=False,
                airplanes=True, slot_name='14', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(-16668.584157662, 44920.392109471), large=False, heli=False,
                airplanes=True, slot_name='13', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(-16676.357421875, 44994.4765625), large=False, heli=False,
                airplanes=True, slot_name='18', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(-16804.998046875, 45454.98046875), large=False, heli=False,
                airplanes=True, slot_name='32', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(-16803.26171875, 45494.9609375), large=False, heli=False,
                airplanes=True, slot_name='33', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(-16865.859375, 45763.0859375), large=False, heli=True,
                airplanes=True, slot_name='37', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(-16811.2890625, 45671.34765625), large=False, heli=False,
                airplanes=True, slot_name='34', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(-16821.287109375, 45690.41015625), large=False, heli=False,
                airplanes=True, slot_name='35', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(-16774.31640625, 45083.71484375), large=False, heli=True,
                airplanes=True, slot_name='19', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(-16793.97265625, 45099.703125), large=False, heli=True,
                airplanes=True, slot_name='20', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(-16809.5078125, 45117.296875), large=False, heli=True,
                airplanes=True, slot_name='21', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(-16818.837890625, 45140.3046875), large=False, heli=True,
                airplanes=True, slot_name='22', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(-17215.724609375, 44984.875), large=False, heli=True,
                airplanes=True, slot_name='23', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(-16821.943359375, 45192.65625), large=False, heli=True,
                airplanes=True, slot_name='24', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(-16821.212890625, 45220.6796875), large=False, heli=True,
                airplanes=True, slot_name='25', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(-16820.3125, 45247.28125), large=False, heli=True,
                airplanes=True, slot_name='26', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(-16819.59765625, 45274.015625), large=False, heli=True,
                airplanes=True, slot_name='27', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=31, position=mapping.Point(-16819.244140625, 45299.265625), large=False, heli=True,
                airplanes=True, slot_name='28', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=32, position=mapping.Point(-16818.4609375, 45326.1328125), large=False, heli=True,
                airplanes=True, slot_name='29', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=33, position=mapping.Point(-16818.9921875, 45350.8984375), large=False, heli=True,
                airplanes=True, slot_name='30', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=34, position=mapping.Point(-16818.4609375, 45376.74609375), large=False, heli=True,
                airplanes=True, slot_name='31', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=35, position=mapping.Point(-16827.734375, 45737.41015625), large=False, heli=False,
                airplanes=True, slot_name='36', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=36, position=mapping.Point(-16854.868761609, 44801.815367799), large=False, heli=False,
                airplanes=True, slot_name='08', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=37, position=mapping.Point(-16825.925056084, 44795.365110598), large=False, heli=False,
                airplanes=True, slot_name='09', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=38, position=mapping.Point(-16886.608270818, 44808.178335765), large=False, heli=False,
                airplanes=True, slot_name='07', length=20.5, width=14.5, height=8.0, shelter=False))


class Dunkirk_Mardyck(Airport):
    id = 4
    name = "Dunkirk Mardyck"
    position = mapping.Point(16494.189453, 46950.742188)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Dunkirk_Mardyck, self).__init__()

        self.runways.append(Runway(260))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(16620.982421875, 47046.9921875), large=False, heli=True,
                airplanes=True, slot_name='01', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(16787.392578125, 47593.80078125), large=False, heli=True,
                airplanes=True, slot_name='03', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(16972.494140625, 48500.3828125), large=False, heli=True,
                airplanes=True, slot_name='11', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(16976.068359375, 48551.44921875), large=False, heli=True,
                airplanes=True, slot_name='10', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(17037.724609375, 48569.19921875), large=False, heli=True,
                airplanes=True, slot_name='09', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(17033.396484375, 48517.35546875), large=False, heli=True,
                airplanes=True, slot_name='08', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(17111.234375, 48471.1015625), large=False, heli=True,
                airplanes=True, slot_name='06', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(17104.76171875, 48412.56640625), large=False, heli=True,
                airplanes=True, slot_name='05', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(17087.908203125, 48255.24609375), large=False, heli=True,
                airplanes=True, slot_name='04', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(16758.31640625, 48566.203125), large=False, heli=True,
                airplanes=True, slot_name='12', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(15870.676757813, 47370.9140625), large=False, heli=True,
                airplanes=True, slot_name='19', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(16001.8515625, 47390.39453125), large=False, heli=True,
                airplanes=True, slot_name='15', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(16003.302734375, 47201.85546875), large=False, heli=True,
                airplanes=True, slot_name='17', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(16002.684570313, 47140.2578125), large=False, heli=True,
                airplanes=True, slot_name='18', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(16124.993164063, 47287.0078125), large=False, heli=True,
                airplanes=True, slot_name='14', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(16137.262695313, 46960.53125), large=False, heli=True,
                airplanes=True, slot_name='13', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(17033.4609375, 48436.95703125), large=False, heli=True,
                airplanes=True, slot_name='07', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(16002.551757812, 47291.11328125), large=False, heli=True,
                airplanes=True, slot_name='16', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(16641.48828125, 47155.7890625), large=False, heli=True,
                airplanes=True, slot_name='02', length=20.5, width=14.5, height=8.0, shelter=False))


class Manston(Airport):
    id = 5
    name = "Manston"
    position = mapping.Point(52264.28125, -15827.487793)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Manston, self).__init__()

        self.runways.append(Runway(100))
        self.runways.append(Runway(40))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(53406.3125, -14998.573242188), large=False, heli=False,
                airplanes=True, slot_name='68', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(53203.453125, -15266.77734375), large=False, heli=False,
                airplanes=True, slot_name='65', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(52945.296875, -15751.91015625), large=False, heli=False,
                airplanes=True, slot_name='57', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(52928.75, -15769.467773437), large=False, heli=False,
                airplanes=True, slot_name='56', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(52752.9296875, -15980.0546875), large=False, heli=False,
                airplanes=True, slot_name='50', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(53156.59375, -17053.43359375), large=False, heli=False,
                airplanes=True, slot_name='29', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(53312.48828125, -17081.896484375), large=False, heli=False,
                airplanes=True, slot_name='28', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(52365.48046875, -17052.736328125), large=False, heli=True,
                airplanes=True, slot_name='12', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(52323.21875, -16874.927734375), large=False, heli=True,
                airplanes=True, slot_name='11', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(52283.64453125, -16696.306640625), large=False, heli=True,
                airplanes=True, slot_name='10', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(52156.890625, -16102.392578125), large=False, heli=True,
                airplanes=True, slot_name='09', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(52121.296875, -15947.27734375), large=False, heli=True,
                airplanes=True, slot_name='08', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(52086.734375, -15785.474609375), large=False, heli=True,
                airplanes=True, slot_name='07', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(52052.79296875, -15624.427734375), large=False, heli=True,
                airplanes=True, slot_name='06', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(52014.1015625, -15459.830078125), large=False, heli=True,
                airplanes=True, slot_name='05', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(51978.41796875, -15306.775390625), large=False, heli=True,
                airplanes=True, slot_name='04', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(51907.2421875, -14972.000976563), large=False, heli=True,
                airplanes=True, slot_name='03', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(51853.65625, -14719.455078125), large=False, heli=True,
                airplanes=True, slot_name='02', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(51806.7734375, -14491.458984375), large=False, heli=True,
                airplanes=True, slot_name='01', length=78.722809, width=67.096466, height=18.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(52943.3359375, -15855.366210938), large=False, heli=True,
                airplanes=True, slot_name='55', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(52924.890625, -15874.98046875), large=False, heli=True,
                airplanes=True, slot_name='54', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(52871.875, -15943.8828125), large=False, heli=True,
                airplanes=True, slot_name='53', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(52859.26953125, -15963.577148438), large=False, heli=True,
                airplanes=True, slot_name='51', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(52734.78515625, -16222.831054688), large=False, heli=True,
                airplanes=True, slot_name='49', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(52710.4765625, -16243.317382813), large=False, heli=True,
                airplanes=True, slot_name='48', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(52685.48828125, -16263.963867187), large=False, heli=True,
                airplanes=True, slot_name='47', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(52660.27734375, -16286.13671875), large=False, heli=True,
                airplanes=True, slot_name='46', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(52632.6640625, -16309.178710937), large=False, heli=True,
                airplanes=True, slot_name='45', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=31, position=mapping.Point(52605.9375, -16330.698242188), large=False, heli=True,
                airplanes=True, slot_name='44', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=32, position=mapping.Point(52691.00390625, -16067.98046875), large=False, heli=True,
                airplanes=True, slot_name='43', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=33, position=mapping.Point(52668.69140625, -16079.984375), large=False, heli=True,
                airplanes=True, slot_name='42', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=34, position=mapping.Point(53383.01171875, -17257.74609375), large=False, heli=True,
                airplanes=True, slot_name='13', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=35, position=mapping.Point(53354.6015625, -17253.74609375), large=False, heli=True,
                airplanes=True, slot_name='14', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=36, position=mapping.Point(53325.6171875, -17248.130859375), large=False, heli=True,
                airplanes=True, slot_name='15', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=37, position=mapping.Point(53277.57421875, -17240.888671875), large=False, heli=True,
                airplanes=True, slot_name='16', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=38, position=mapping.Point(53244.83203125, -17234.91015625), large=False, heli=True,
                airplanes=True, slot_name='17', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=39, position=mapping.Point(53210.19921875, -17228.7421875), large=False, heli=True,
                airplanes=True, slot_name='18', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=40, position=mapping.Point(53174.734375, -17222.40625), large=False, heli=True,
                airplanes=True, slot_name='19', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=41, position=mapping.Point(53140.9453125, -17216.03515625), large=False, heli=True,
                airplanes=True, slot_name='20', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=42, position=mapping.Point(53105.46875, -17209.234375), large=False, heli=True,
                airplanes=True, slot_name='21', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=43, position=mapping.Point(53069.45703125, -17202.203125), large=False, heli=True,
                airplanes=True, slot_name='22', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=44, position=mapping.Point(53035.56640625, -17195.953125), large=False, heli=True,
                airplanes=True, slot_name='23', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=45, position=mapping.Point(53000.06640625, -17189.93359375), large=False, heli=True,
                airplanes=True, slot_name='24', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=46, position=mapping.Point(52964.078125, -17183.412109375), large=False, heli=True,
                airplanes=True, slot_name='25', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=47, position=mapping.Point(52929.88671875, -17177.169921875), large=False, heli=True,
                airplanes=True, slot_name='26', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=48, position=mapping.Point(52896.27734375, -17170.6484375), large=False, heli=True,
                airplanes=True, slot_name='27', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=49, position=mapping.Point(53123.01953125, -17070.728515625), large=False, heli=True,
                airplanes=True, slot_name='30', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=50, position=mapping.Point(53090.97265625, -17088.548828125), large=False, heli=True,
                airplanes=True, slot_name='31', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=51, position=mapping.Point(53061.2421875, -17103.8671875), large=False, heli=True,
                airplanes=True, slot_name='32', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=52, position=mapping.Point(53033.53515625, -17118.177734375), large=False, heli=True,
                airplanes=True, slot_name='33', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=53, position=mapping.Point(52641.765625, -16708.86328125), large=False, heli=True,
                airplanes=True, slot_name='36', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=54, position=mapping.Point(52649.06640625, -16742.412109375), large=False, heli=True,
                airplanes=True, slot_name='35', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=55, position=mapping.Point(53036.67578125, -15700.387695313), large=False, heli=True,
                airplanes=True, slot_name='58', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=56, position=mapping.Point(53040.984375, -15654.38671875), large=False, heli=False,
                airplanes=True, slot_name='59', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=57, position=mapping.Point(53171.796875, -15473.270507813), large=False, heli=True,
                airplanes=True, slot_name='63', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=58, position=mapping.Point(53106.578125, -15625.421875), large=False, heli=True,
                airplanes=True, slot_name='60', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=59, position=mapping.Point(53109.8203125, -15563.850585938), large=False, heli=False,
                airplanes=True, slot_name='61', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=60, position=mapping.Point(53163.23046875, -15388.58984375), large=False, heli=False,
                airplanes=True, slot_name='64', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=61, position=mapping.Point(53160.6640625, -15522.41015625), large=False, heli=True,
                airplanes=True, slot_name='62', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=62, position=mapping.Point(53248.34375, -15219.190429687), large=False, heli=True,
                airplanes=True, slot_name='66', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=63, position=mapping.Point(53272.7421875, -15174.526367187), large=False, heli=True,
                airplanes=True, slot_name='67', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=64, position=mapping.Point(52626.15625, -16639.712890625), large=False, heli=True,
                airplanes=True, slot_name='38', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=65, position=mapping.Point(52633.453125, -16673.265625), large=False, heli=True,
                airplanes=True, slot_name='37', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=66, position=mapping.Point(52611.10546875, -16571.271484375), large=False, heli=True,
                airplanes=True, slot_name='40', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=67, position=mapping.Point(52618.71875, -16604.5390625), large=False, heli=True,
                airplanes=True, slot_name='39', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=68, position=mapping.Point(52665.2890625, -16780.703125), large=False, heli=True,
                airplanes=True, slot_name='34', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=69, position=mapping.Point(52590.48046875, -16524.296875), large=False, heli=True,
                airplanes=True, slot_name='41', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=70, position=mapping.Point(52826.796875, -15893.015625), large=False, heli=False,
                airplanes=True, slot_name='52', length=20.5, width=14.5, height=8.0, shelter=False))


class Hawkinge(Airport):
    id = 6
    name = "Hawkinge"
    position = mapping.Point(27031.420898, -29403.830078)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Hawkinge, self).__init__()

        self.runways.append(Runway(190))
        self.runways.append(Runway(40))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(27040, -30028.244140625), large=False, heli=False,
                airplanes=True, slot_name='10', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(27308.40234375, -29087.111328125), large=False, heli=True,
                airplanes=True, slot_name='15', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(27276.8046875, -29072.44140625), large=False, heli=True,
                airplanes=True, slot_name='16', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(27244.27734375, -29057.19140625), large=False, heli=True,
                airplanes=True, slot_name='17', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(27211.73046875, -29041.603515625), large=False, heli=True,
                airplanes=True, slot_name='18', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(27420.6953125, -29182.8515625), large=False, heli=False,
                airplanes=True, slot_name='14', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(27200.55078125, -29937.79296875), large=False, heli=False,
                airplanes=True, slot_name='11', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(26932.28125, -30194.1953125), large=False, heli=False,
                airplanes=True, slot_name='08', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(27140.705078125, -29000.99609375), large=False, heli=False,
                airplanes=True, slot_name='19', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(26591.95703125, -30091.30078125), large=False, heli=True,
                airplanes=True, slot_name='03', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(26573.298828125, -30068.294921875), large=False, heli=True,
                airplanes=True, slot_name='02', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(26647.7734375, -30132.556640625), large=False, heli=True,
                airplanes=True, slot_name='05', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(26631.279296875, -30108.376953125), large=False, heli=True,
                airplanes=True, slot_name='04', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(26724.1640625, -30207.951171875), large=False, heli=True,
                airplanes=True, slot_name='07', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(26699.814453125, -30192.974609375), large=False, heli=True,
                airplanes=True, slot_name='06', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(26771.0234375, -29008.568359375), large=False, heli=True,
                airplanes=True, slot_name='25', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(26748.544921875, -29027.4140625), large=False, heli=True,
                airplanes=True, slot_name='26', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(26802.212890625, -28970.296875), large=False, heli=True,
                airplanes=True, slot_name='24', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(26818.9296875, -28946.75), large=False, heli=True,
                airplanes=True, slot_name='23', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(26838.42578125, -28883.888671875), large=False, heli=True,
                airplanes=True, slot_name='22', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(26856.09765625, -28859.921875), large=False, heli=True,
                airplanes=True, slot_name='21', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(27071.71875, -28954.021484375), large=False, heli=False,
                airplanes=True, slot_name='20', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(27074.619140625, -30069.642578125), large=False, heli=True,
                airplanes=True, slot_name='09', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(26579.890625, -29575.828125), large=False, heli=True,
                airplanes=True, slot_name='01', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(27157.048828125, -29923.697265625), large=False, heli=True,
                airplanes=True, slot_name='12', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(27139.091796875, -29903.3515625), large=False, heli=True,
                airplanes=True, slot_name='13', length=26.0, width=22.0, height=11.0, shelter=False))


class Lympne(Airport):
    id = 7
    name = "Lympne"
    position = mapping.Point(23796.457031, -39541.675781)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Lympne, self).__init__()

        self.runways.append(Runway(130))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=0, position=mapping.Point(23345.69140625, -39134.625), large=False, heli=True,
                airplanes=True, slot_name='22', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=1, position=mapping.Point(23201.74609375, -39010.74609375), large=False, heli=True,
                airplanes=True, slot_name='21', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(23783.634765625, -39796.25390625), large=False, heli=True,
                airplanes=True, slot_name='04', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(23748.3203125, -39728.65234375), large=False, heli=True,
                airplanes=True, slot_name='05', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(23686.923828125, -39705.84765625), large=False, heli=True,
                airplanes=True, slot_name='06', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(23644.4921875, -39668.9140625), large=False, heli=True,
                airplanes=True, slot_name='07', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(23591.3515625, -39682), large=False, heli=True,
                airplanes=True, slot_name='08', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(23516.357421875, -39700.60546875), large=False, heli=True,
                airplanes=True, slot_name='09', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(23471.884765625, -39712.6328125), large=False, heli=True,
                airplanes=True, slot_name='10', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(23254.34765625, -39557.3359375), large=False, heli=True,
                airplanes=True, slot_name='11', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(23255.8984375, -39531.75390625), large=False, heli=True,
                airplanes=True, slot_name='12', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(23251.603515625, -39492.6484375), large=False, heli=True,
                airplanes=True, slot_name='13', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(23246.7109375, -39461.828125), large=False, heli=True,
                airplanes=True, slot_name='14', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(23244.21875, -39432.16796875), large=False, heli=True,
                airplanes=True, slot_name='15', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(23241.810546875, -39402.59375), large=False, heli=True,
                airplanes=True, slot_name='16', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(23239.392578125, -39372.625), large=False, heli=True,
                airplanes=True, slot_name='17', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(23236.884765625, -39342.32421875), large=False, heli=True,
                airplanes=True, slot_name='18', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(23234.56640625, -39312.13671875), large=False, heli=True,
                airplanes=True, slot_name='19', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(23232.04296875, -39281.43359375), large=False, heli=True,
                airplanes=True, slot_name='20', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(23823.890625, -39832.484375), large=False, heli=True,
                airplanes=True, slot_name='03', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(23863.69140625, -39868.34375), large=False, heli=True,
                airplanes=True, slot_name='02', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(23894.69140625, -39896.23046875), large=False, heli=True,
                airplanes=True, slot_name='01', length=26.0, width=22.0, height=11.0, shelter=False))


class Detling(Airport):
    id = 8
    name = "Detling"
    position = mapping.Point(49629.695313, -67886.082031)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(Detling, self).__init__()

        self.runways.append(Runway(230))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=2, position=mapping.Point(50092, -67870.7734375), large=False, heli=False,
                airplanes=True, slot_name='21', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=3, position=mapping.Point(49994.62109375, -67868.7890625), large=False, heli=True,
                airplanes=True, slot_name='13', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(49976.59375, -67835.890625), large=False, heli=True,
                airplanes=True, slot_name='14', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(49966.203125, -67798.328125), large=False, heli=True,
                airplanes=True, slot_name='15', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(49965.953125, -67758.3125), large=False, heli=True,
                airplanes=True, slot_name='16', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(49974.75, -67717.578125), large=False, heli=True,
                airplanes=True, slot_name='17', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(49988.5859375, -67680.0625), large=False, heli=True,
                airplanes=True, slot_name='18', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(50083.7734375, -67595.6328125), large=False, heli=True,
                airplanes=True, slot_name='22', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(50069.91796875, -67573.203125), large=False, heli=True,
                airplanes=True, slot_name='23', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(50055.5078125, -67551.859375), large=False, heli=True,
                airplanes=True, slot_name='24', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(49988.234375, -67435.2890625), large=False, heli=True,
                airplanes=True, slot_name='25', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(50009.2421875, -67892.328125), large=False, heli=True,
                airplanes=True, slot_name='12', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(50020.46875, -67911.6484375), large=False, heli=True,
                airplanes=True, slot_name='11', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(50031.15625, -67930.6796875), large=False, heli=True,
                airplanes=True, slot_name='10', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(50042.63671875, -67947.390625), large=False, heli=True,
                airplanes=True, slot_name='09', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(50053.08203125, -67964.3203125), large=False, heli=True,
                airplanes=True, slot_name='08', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(50063.7109375, -67981.5546875), large=False, heli=True,
                airplanes=True, slot_name='07', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(50111.7265625, -67951.90625), large=False, heli=True,
                airplanes=True, slot_name='19', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(50099.87890625, -67932.140625), large=False, heli=True,
                airplanes=True, slot_name='20', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(49863.71484375, -67428.3828125), large=False, heli=True,
                airplanes=True, slot_name='29', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(49883.0390625, -67414.96875), large=False, heli=True,
                airplanes=True, slot_name='28', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(49884.2734375, -67527.0859375), large=False, heli=True,
                airplanes=True, slot_name='32', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(49864.43359375, -67539.1953125), large=False, heli=True,
                airplanes=True, slot_name='31', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(49847.52734375, -67549.3984375), large=False, heli=True,
                airplanes=True, slot_name='30', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(50034.46484375, -68246.7890625), large=False, heli=True,
                airplanes=True, slot_name='06', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(50013.67578125, -68263.84375), large=False, heli=True,
                airplanes=True, slot_name='05', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(49993.0859375, -68280.703125), large=False, heli=True,
                airplanes=True, slot_name='04', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(49971.359375, -68298.5234375), large=False, heli=True,
                airplanes=True, slot_name='03', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(49946.34375, -68319.171875), large=False, heli=True,
                airplanes=True, slot_name='02', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=31, position=mapping.Point(49923.33203125, -68337.96875), large=False, heli=True,
                airplanes=True, slot_name='01', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=32, position=mapping.Point(49883.68359375, -67357.6328125), large=False, heli=True,
                airplanes=True, slot_name='27', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=33, position=mapping.Point(49953.921875, -67380.265625), large=False, heli=True,
                airplanes=True, slot_name='26', length=20.5, width=14.5, height=8.0, shelter=False))


class High_Halden(Airport):
    id = 12
    name = "High Halden"
    position = mapping.Point(28992.270508, -62020.105469)
    tacan = None
    unit_zones = []
    civilian = False
    slot_version = 2

    def __init__(self):
        super(High_Halden, self).__init__()

        self.runways.append(Runway(110))
        self.runways.append(Runway(40))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=4, position=mapping.Point(29893.03515625, -60978.26171875), large=False, heli=True,
                airplanes=True, slot_name='56', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=5, position=mapping.Point(29906.265625, -60952.12109375), large=False, heli=False,
                airplanes=True, slot_name='55', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=6, position=mapping.Point(29920.90625, -61047.30078125), large=False, heli=False,
                airplanes=True, slot_name='57', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=7, position=mapping.Point(28669.79296875, -61436.43359375), large=False, heli=True,
                airplanes=True, slot_name='31', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=8, position=mapping.Point(28799.478515625, -61688.06640625), large=False, heli=True,
                airplanes=True, slot_name='29', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=9, position=mapping.Point(28802.302975559, -61717.962317687), large=False, heli=False,
                airplanes=True, slot_name='28', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=10, position=mapping.Point(29030.19921875, -62638.83984375), large=False, heli=True,
                airplanes=True, slot_name='02', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=11, position=mapping.Point(29066.056640625, -62638.703125), large=False, heli=True,
                airplanes=True, slot_name='01', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=12, position=mapping.Point(29006.9453125, -62611.453125), large=False, heli=True,
                airplanes=True, slot_name='03', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=13, position=mapping.Point(29004.22265625, -62554.95703125), large=False, heli=True,
                airplanes=True, slot_name='04', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=14, position=mapping.Point(28907.9375, -61307.63671875), large=False, heli=True,
                airplanes=True, slot_name='32', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=15, position=mapping.Point(28769.376953125, -61631.80078125), large=False, heli=True,
                airplanes=True, slot_name='30', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=16, position=mapping.Point(28823.103515625, -61778.9453125), large=False, heli=True,
                airplanes=True, slot_name='27', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=17, position=mapping.Point(28773.73046875, -61829.390625), large=False, heli=True,
                airplanes=True, slot_name='26', length=26.0, width=22.0, height=11.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=18, position=mapping.Point(28869.740234375, -62005.37890625), large=False, heli=True,
                airplanes=True, slot_name='22', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=19, position=mapping.Point(28850.638671875, -61916.87109375), large=False, heli=True,
                airplanes=True, slot_name='24', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=20, position=mapping.Point(28820.255859375, -61968.77734375), large=False, heli=True,
                airplanes=True, slot_name='23', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=21, position=mapping.Point(28875.26953125, -62033.1484375), large=False, heli=True,
                airplanes=True, slot_name='21', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=22, position=mapping.Point(28823.474609375, -62100.96875), large=False, heli=True,
                airplanes=True, slot_name='19', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=23, position=mapping.Point(28830.435546875, -62064.53125), large=False, heli=True,
                airplanes=True, slot_name='20', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=24, position=mapping.Point(28891.017578125, -62125.796875), large=False, heli=True,
                airplanes=True, slot_name='18', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=25, position=mapping.Point(28817.3359375, -62177.42578125), large=False, heli=True,
                airplanes=True, slot_name='16', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=26, position=mapping.Point(28837.1171875, -62263.390625), large=False, heli=True,
                airplanes=True, slot_name='14', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=27, position=mapping.Point(28929.705078125, -62308.16796875), large=False, heli=True,
                airplanes=True, slot_name='12', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=28, position=mapping.Point(28866.798828125, -62387.12890625), large=False, heli=True,
                airplanes=True, slot_name='10', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=29, position=mapping.Point(28959.255859375, -62436.02734375), large=False, heli=True,
                airplanes=True, slot_name='08', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=30, position=mapping.Point(28904.9375, -62515.3671875), large=False, heli=True,
                airplanes=True, slot_name='06', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=31, position=mapping.Point(28828.00390625, -62143.65625), large=False, heli=True,
                airplanes=True, slot_name='17', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=32, position=mapping.Point(28845.28515625, -62229.82421875), large=False, heli=True,
                airplanes=True, slot_name='15', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=33, position=mapping.Point(28872.94921875, -62354.12109375), large=False, heli=True,
                airplanes=True, slot_name='11', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=34, position=mapping.Point(28912.580078125, -62477.8359375), large=False, heli=True,
                airplanes=True, slot_name='07', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=35, position=mapping.Point(28898.876953125, -62282.7109375), large=False, heli=True,
                airplanes=True, slot_name='13', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=36, position=mapping.Point(28927.14453125, -62403.65625), large=False, heli=True,
                airplanes=True, slot_name='09', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=37, position=mapping.Point(28931.220703125, -62540.4140625), large=False, heli=True,
                airplanes=True, slot_name='05', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=38, position=mapping.Point(28948.283203125, -61277.3359375), large=False, heli=True,
                airplanes=True, slot_name='33', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=39, position=mapping.Point(28994.4609375, -61251.44921875), large=False, heli=True,
                airplanes=True, slot_name='34', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=40, position=mapping.Point(29027.24609375, -61227.625), large=False, heli=True,
                airplanes=True, slot_name='35', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=41, position=mapping.Point(29075.998046875, -61118.1640625), large=False, heli=True,
                airplanes=True, slot_name='37', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=42, position=mapping.Point(29169.4453125, -61086.8203125), large=False, heli=True,
                airplanes=True, slot_name='40', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=43, position=mapping.Point(29235.75, -61101.48828125), large=False, heli=True,
                airplanes=True, slot_name='42', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=44, position=mapping.Point(29280.853515625, -61083.7265625), large=False, heli=True,
                airplanes=True, slot_name='43', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=45, position=mapping.Point(29163.4765625, -61154.9140625), large=False, heli=True,
                airplanes=True, slot_name='39', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=46, position=mapping.Point(29315.845703125, -61060.7734375), large=False, heli=True,
                airplanes=True, slot_name='44', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=47, position=mapping.Point(29338.0859375, -60988.75), large=False, heli=True,
                airplanes=True, slot_name='46', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=48, position=mapping.Point(29441.05078125, -60979.4296875), large=False, heli=True,
                airplanes=True, slot_name='48', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=49, position=mapping.Point(29489.765625, -60955.5546875), large=False, heli=True,
                airplanes=True, slot_name='49', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=50, position=mapping.Point(29521.095703125, -60935.73046875), large=False, heli=True,
                airplanes=True, slot_name='50', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=51, position=mapping.Point(29547.1953125, -60855.515625), large=False, heli=True,
                airplanes=True, slot_name='52', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=52, position=mapping.Point(29623.72265625, -60872.0703125), large=False, heli=True,
                airplanes=True, slot_name='54', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=53, position=mapping.Point(29585.650390625, -60863.25), large=False, heli=True,
                airplanes=True, slot_name='53', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=54, position=mapping.Point(29370.611328125, -60994.6640625), large=False, heli=True,
                airplanes=True, slot_name='47', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=55, position=mapping.Point(29202.431640625, -61084.39453125), large=False, heli=True,
                airplanes=True, slot_name='41', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=56, position=mapping.Point(29106.80078125, -61105.578125), large=False, heli=True,
                airplanes=True, slot_name='38', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=57, position=mapping.Point(29032.36328125, -61165.94140625), large=False, heli=False,
                airplanes=True, slot_name='36', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=58, position=mapping.Point(29314.0703125, -61015.54296875), large=False, heli=True,
                airplanes=True, slot_name='45', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=59, position=mapping.Point(29528.37109375, -60885.8125), large=False, heli=True,
                airplanes=True, slot_name='51', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=60, position=mapping.Point(29067.1328125, -61482.41015625), large=False, heli=True,
                airplanes=True, slot_name='79', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=61, position=mapping.Point(29141.05859375, -61494.5390625), large=False, heli=True,
                airplanes=True, slot_name='77', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=62, position=mapping.Point(29169.58203125, -61414.6953125), large=False, heli=True,
                airplanes=True, slot_name='75', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=63, position=mapping.Point(29274.083984375, -61432.140625), large=False, heli=True,
                airplanes=True, slot_name='73', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=64, position=mapping.Point(29274.0859375, -61357.28125), large=False, heli=True,
                airplanes=True, slot_name='71', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=65, position=mapping.Point(29370.64453125, -61294.6328125), large=False, heli=True,
                airplanes=True, slot_name='70', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=66, position=mapping.Point(29452.283203125, -61308.9375), large=False, heli=True,
                airplanes=True, slot_name='68', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=67, position=mapping.Point(29541.73046875, -61253.55859375), large=False, heli=True,
                airplanes=True, slot_name='65', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=68, position=mapping.Point(29558.8828125, -61184.20703125), large=False, heli=True,
                airplanes=True, slot_name='63', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=69, position=mapping.Point(29694.529296875, -61159.4375), large=False, heli=True,
                airplanes=True, slot_name='61', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=70, position=mapping.Point(29767.5703125, -61057.1875), large=False, heli=True,
                airplanes=True, slot_name='59', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=71, position=mapping.Point(29807.02734375, -61028.734375), large=False, heli=True,
                airplanes=True, slot_name='58', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=72, position=mapping.Point(29665.083984375, -61155.9296875), large=False, heli=True,
                airplanes=True, slot_name='62', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=73, position=mapping.Point(29509.04296875, -61253.921875), large=False, heli=True,
                airplanes=True, slot_name='66', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=74, position=mapping.Point(29421.974609375, -61303.3828125), large=False, heli=True,
                airplanes=True, slot_name='69', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=75, position=mapping.Point(29232.9765625, -61422.91796875), large=False, heli=True,
                airplanes=True, slot_name='74', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=76, position=mapping.Point(29103.5546875, -61489.796875), large=False, heli=True,
                airplanes=True, slot_name='78', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=77, position=mapping.Point(29159.43359375, -61465.609375), large=False, heli=True,
                airplanes=True, slot_name='76', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=78, position=mapping.Point(29295.35546875, -61403.2421875), large=False, heli=True,
                airplanes=True, slot_name='72', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=79, position=mapping.Point(29448.609375, -61263.4140625), large=False, heli=True,
                airplanes=True, slot_name='67', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=80, position=mapping.Point(29559.1640625, -61222.90625), large=False, heli=True,
                airplanes=True, slot_name='64', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=81, position=mapping.Point(29693.294921875, -61115.0390625), large=False, heli=True,
                airplanes=True, slot_name='60', length=20.5, width=14.5, height=8.0, shelter=False))
        self.parking_slots.append(ParkingSlot(
                crossroad_idx=82, position=mapping.Point(28847.181640625, -61891.40625), large=False, heli=True,
                airplanes=True, slot_name='25', length=20.5, width=14.5, height=8.0, shelter=False))


class TheChannel(Terrain):
    center = {"lat": 50.875, "long": 1.5875}
    bounds = mapping.Rectangle(74967, -114995, -129982, 129991)
    map_view_default = MapView(mapping.Point(0, 0), 1000000)
    city_graph = None
    temperature = [
        (-10, 10),
        (-9, 10),
        (-3, 12),
        (-1, 14),
        (0, 18),
        (2, 22),
        (7, 30),
        (8, 32),
        (3, 28),
        (0, 22),
        (-2, 16),
        (-8, 10)
    ]
    assert (len(temperature) == 12)

    def __init__(self):
        super(TheChannel, self).__init__("TheChannel")
        self.bullseye_blue = {"x": 0, "y": 0}
        self.bullseye_red = {"x": 0, "y": 0}

        self.airports['Abbeville Drucat'] = Abbeville_Drucat()
        self.airports['Merville Calonne'] = Merville_Calonne()
        self.airports['Saint Omer Longuenesse'] = Saint_Omer_Longuenesse()
        self.airports['Dunkirk Mardyck'] = Dunkirk_Mardyck()
        self.airports['Manston'] = Manston()
        self.airports['Hawkinge'] = Hawkinge()
        self.airports['Lympne'] = Lympne()
        self.airports['Detling'] = Detling()
        self.airports['High Halden'] = High_Halden()

    def abbeville_drucat(self) -> Airport:
        return self.airports["Abbeville Drucat"]

    def merville_calonne(self) -> Airport:
        return self.airports["Merville Calonne"]

    def saint_omer_longuenesse(self) -> Airport:
        return self.airports["Saint Omer Longuenesse"]

    def dunkirk_mardyck(self) -> Airport:
        return self.airports["Dunkirk Mardyck"]

    def manston(self) -> Airport:
        return self.airports["Manston"]

    def hawkinge(self) -> Airport:
        return self.airports["Hawkinge"]

    def lympne(self) -> Airport:
        return self.airports["Lympne"]

    def detling(self) -> Airport:
        return self.airports["Detling"]

    def high_halden(self) -> Airport:
        return self.airports["High Halden"]


# This file is generated from pydcs_export.lua

from . import unittype


class Armed_speedboat(unittype.ShipType):
    id = "speedboat"
    name = "Armed speedboat"
    detection_range = 3000
    threat_range = 1000
    air_weapon_dist = 1000


class CVN_70_Carl_Vinson(unittype.ShipType):
    id = "VINSON"
    name = "CVN-70 Carl Vinson"
    plane_num = 72
    helicopter_num = 6
    parking = 4
    detection_range = 30000
    threat_range = 15000
    air_weapon_dist = 15000


class Oliver_Hazzard_Perry_class(unittype.ShipType):
    id = "PERRY"
    name = "Oliver Hazzard Perry class"
    helicopter_num = 2
    parking = 1
    detection_range = 150000
    threat_range = 100000
    air_weapon_dist = 100000


class Ticonderoga_class(unittype.ShipType):
    id = "TICONDEROG"
    name = "Ticonderoga class"
    helicopter_num = 2
    parking = 1
    detection_range = 150000
    threat_range = 100000
    air_weapon_dist = 100000


class FFL_1124_4_Grisha(unittype.ShipType):
    id = "ALBATROS"
    name = "FFL 1124.4 Grisha"
    detection_range = 30000
    threat_range = 16000
    air_weapon_dist = 16000


class CV_1143_5_Admiral_Kuznetsov(unittype.ShipType):
    id = "KUZNECOW"
    name = "CV 1143.5 Admiral Kuznetsov"
    plane_num = 24
    helicopter_num = 12
    parking = 3
    detection_range = 25000
    threat_range = 12000
    air_weapon_dist = 12000


class FSG_1241_1MP_Molniya(unittype.ShipType):
    id = "MOLNIYA"
    name = "FSG 1241.1MP Molniya"
    detection_range = 21000
    threat_range = 2000
    air_weapon_dist = 2000


class CG_1164_Moskva(unittype.ShipType):
    id = "MOSCOW"
    name = "CG 1164 Moskva"
    helicopter_num = 1
    parking = 1
    detection_range = 160000
    threat_range = 75000
    air_weapon_dist = 75000


class FFG_11540_Neustrashimy(unittype.ShipType):
    id = "NEUSTRASH"
    name = "FFG 11540 Neustrashimy"
    helicopter_num = 1
    parking = 1
    detection_range = 27000
    threat_range = 12000
    air_weapon_dist = 12000


class CGN_1144_2_Pyotr_Velikiy(unittype.ShipType):
    id = "PIOTR"
    name = "CGN 1144.2 Pyotr Velikiy"
    helicopter_num = 1
    parking = 1
    detection_range = 250000
    threat_range = 190000
    air_weapon_dist = 190000


class FF_1135M_Rezky(unittype.ShipType):
    id = "REZKY"
    name = "FF 1135M Rezky"
    detection_range = 30000
    threat_range = 16000
    air_weapon_dist = 16000


class Tanker_Elnya_160(unittype.ShipType):
    id = "ELNYA"
    name = "Tanker Elnya 160"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class Dry_cargo_ship_Ivanov(unittype.ShipType):
    id = "Dry-cargo ship-2"
    name = "Dry cargo ship Ivanov"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class Bulk_cargo_ship_Yakushev(unittype.ShipType):
    id = "Dry-cargo ship-1"
    name = "Bulk cargo ship Yakushev"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class Civil_boat_Zvezdny(unittype.ShipType):
    id = "ZWEZDNY"
    name = "Civil boat Zvezdny"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class SSK_877(unittype.ShipType):
    id = "KILO"
    name = "SSK 877"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class SSK_641B(unittype.ShipType):
    id = "SOM"
    name = "SSK 641B"
    detection_range = 0
    threat_range = 0
    air_weapon_dist = 0


class CV_1143_5_Admiral_Kuznetsov_2017(unittype.ShipType):
    id = "CV_1143_5"
    name = "CV 1143.5 Admiral Kuznetsov(2017)"
    plane_num = 24
    helicopter_num = 12
    parking = 3
    detection_range = 25000
    threat_range = 12000
    air_weapon_dist = 12000


class CVN_74_John_C__Stennis(unittype.ShipType):
    id = "Stennis"
    name = "CVN-74 John C. Stennis"
    plane_num = 72
    helicopter_num = 6
    parking = 4
    detection_range = 30000
    threat_range = 15000
    air_weapon_dist = 15000


class CVN_71_Theodore_Roosevelt(unittype.ShipType):
    id = "CVN_71"
    name = "CVN-71 Theodore Roosevelt"
    plane_num = 72
    helicopter_num = 6
    parking = 4
    detection_range = 30000
    threat_range = 15000
    air_weapon_dist = 15000


class CVN_72_Abraham_Lincoln(unittype.ShipType):
    id = "CVN_72"
    name = "CVN-72 Abraham Lincoln"
    plane_num = 72
    helicopter_num = 6
    parking = 4
    detection_range = 30000
    threat_range = 15000
    air_weapon_dist = 15000


class CVN_73_George_Washington(unittype.ShipType):
    id = "CVN_73"
    name = "CVN-73 George Washington"
    plane_num = 72
    helicopter_num = 6
    parking = 4
    detection_range = 30000
    threat_range = 15000
    air_weapon_dist = 15000


class USS_Arleigh_Burke_IIa(unittype.ShipType):
    id = "USS_Arleigh_Burke_IIa"
    name = "USS Arleigh Burke IIa"
    helicopter_num = 2
    parking = 1
    detection_range = 150000
    threat_range = 100000
    air_weapon_dist = 100000


class LHA_1_Tarawa(unittype.ShipType):
    id = "LHA_Tarawa"
    name = "LHA-1 Tarawa"
    plane_num = 40
    helicopter_num = 36
    parking = 4
    detection_range = 300000
    threat_range = 150000
    air_weapon_dist = 150000


class Type_052B_Destroyer(unittype.ShipType):
    id = "Type_052B"
    name = "Type 052B Destroyer"
    plane_num = 0
    helicopter_num = 1
    parking = 1
    detection_range = 100000
    threat_range = 30000
    air_weapon_dist = 30000


class Type_054A_Frigate(unittype.ShipType):
    id = "Type_054A"
    name = "Type 054A Frigate"
    plane_num = 0
    helicopter_num = 1
    parking = 1
    detection_range = 160000
    threat_range = 45000
    air_weapon_dist = 45000


class Type_052C_Destroyer(unittype.ShipType):
    id = "Type_052C"
    name = "Type 052C Destroyer"
    plane_num = 0
    helicopter_num = 1
    parking = 1
    detection_range = 260000
    threat_range = 100000
    air_weapon_dist = 100000


class Type_093_Attack_Submarine(unittype.ShipType):
    id = "Type_093"
    name = "Type 093 Attack Submarine"
    detection_range = 40000
    threat_range = 40000
    air_weapon_dist = 30000


class Type_071_Amphibious_Transport_Dock(unittype.ShipType):
    id = "Type_071"
    name = "Type 071 Amphibious Transport Dock"
    helicopter_num = 2
    parking = 2
    detection_range = 300000
    threat_range = 150000
    air_weapon_dist = 150000


class LST_Mk_II(unittype.ShipType):
    id = "LST_Mk2"
    name = "LST Mk.II"
    detection_range = 0
    threat_range = 4000
    air_weapon_dist = 4000


class LS_Samuel_Chase(unittype.ShipType):
    id = "USS_Samuel_Chase"
    name = "LS Samuel Chase"
    detection_range = 0
    threat_range = 7000
    air_weapon_dist = 7000


class LCVP__Higgins_boat(unittype.ShipType):
    id = "Higgins_boat"
    name = "LCVP (Higgins boat)"
    detection_range = 3000
    threat_range = 1000
    air_weapon_dist = 1000


class Uboat_VIIC_U_flak(unittype.ShipType):
    id = "Uboat_VIIC"
    name = "Uboat VIIC U-flak"
    detection_range = 12000
    threat_range = 4000
    air_weapon_dist = 4000


class Schnellboot_type_S130(unittype.ShipType):
    id = "Schnellboot_type_S130"
    name = "Schnellboot type S130"
    detection_range = 12000
    threat_range = 7000
    air_weapon_dist = 7000

ship_map = {
    "speedboat": Armed_speedboat,
    "VINSON": CVN_70_Carl_Vinson,
    "PERRY": Oliver_Hazzard_Perry_class,
    "TICONDEROG": Ticonderoga_class,
    "ALBATROS": FFL_1124_4_Grisha,
    "KUZNECOW": CV_1143_5_Admiral_Kuznetsov,
    "MOLNIYA": FSG_1241_1MP_Molniya,
    "MOSCOW": CG_1164_Moskva,
    "NEUSTRASH": FFG_11540_Neustrashimy,
    "PIOTR": CGN_1144_2_Pyotr_Velikiy,
    "REZKY": FF_1135M_Rezky,
    "ELNYA": Tanker_Elnya_160,
    "Dry-cargo ship-2": Dry_cargo_ship_Ivanov,
    "Dry-cargo ship-1": Bulk_cargo_ship_Yakushev,
    "ZWEZDNY": Civil_boat_Zvezdny,
    "KILO": SSK_877,
    "SOM": SSK_641B,
    "CV_1143_5": CV_1143_5_Admiral_Kuznetsov_2017,
    "Stennis": CVN_74_John_C__Stennis,
    "CVN_71": CVN_71_Theodore_Roosevelt,
    "CVN_72": CVN_72_Abraham_Lincoln,
    "CVN_73": CVN_73_George_Washington,
    "USS_Arleigh_Burke_IIa": USS_Arleigh_Burke_IIa,
    "LHA_Tarawa": LHA_1_Tarawa,
    "Type_052B": Type_052B_Destroyer,
    "Type_054A": Type_054A_Frigate,
    "Type_052C": Type_052C_Destroyer,
    "Type_093": Type_093_Attack_Submarine,
    "Type_071": Type_071_Amphibious_Transport_Dock,
    "LST_Mk2": LST_Mk_II,
    "USS_Samuel_Chase": LS_Samuel_Chase,
    "Higgins_boat": LCVP__Higgins_boat,
    "Uboat_VIIC": Uboat_VIIC_U_flak,
    "Schnellboot_type_S130": Schnellboot_type_S130,
}



"""
This utility classes provides methods to check players installed DCS environment.

TODO : add method 'is_using_open_beta', 'is_using_stable'
TODO : [NICE to have] add method to check list of installed DCS modules (could be done either through windows registry, or through filesystem analysis ?)
"""

import os

is_windows_os = True
try:
    import winreg
except ImportError:
    is_windows_os = False
    winreg = None
    print("WARNING : Trying to run pydcs on non Windows machine")


def is_using_dcs_steam_edition():
    """
    Check if DCS World : Steam Edition version is installed on this computer
    :return True if DCS Steam edition is installed,
            -1 if DCS Steam Edition is registered in Steam apps but not installed,
            False if never installed in Steam
    """
    if not is_windows_os:
        return False
    try:
        # Note : Steam App ID for DCS World is 223750
        dcs_steam_app_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Valve\\Steam\\Apps\\223750")
        installed = winreg.QueryValueEx(dcs_steam_app_key, "Installed")
        winreg.CloseKey(dcs_steam_app_key)
        if installed[0] == 1:
            return True
        else:
            return False
    except FileNotFoundError as fnfe:
        return False


def is_using_dcs_standalone_edition():
    """
    Check if DCS World standalone edition is installed on this computer
    :return True if Standalone is installed, False if it is not
    """
    if not is_windows_os:
        return False
    try:
        dcs_path_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Eagle Dynamics\\DCS World")
        winreg.CloseKey(dcs_path_key)
        return True
    except FileNotFoundError as fnfe:
        try:
            dcs_path_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Eagle Dynamics\\DCS World OpenBeta")
            winreg.CloseKey(dcs_path_key)
            return True
        except FileNotFoundError:
            return False


def get_dcs_install_directory():
    """
    Get the DCS World install directory for this computer
    :return DCS World install directory
    """
    if is_using_dcs_standalone_edition():
        try:
            dcs_path_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Eagle Dynamics\\DCS World")
            path = winreg.QueryValueEx(dcs_path_key, "Path")
            dcs_dir = path[0] + os.path.sep
            winreg.CloseKey(dcs_path_key)
            return dcs_dir
        except FileNotFoundError as fnfe:
            try:
                dcs_path_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Eagle Dynamics\\DCS World OpenBeta")
                path = winreg.QueryValueEx(dcs_path_key, "Path")
                dcs_dir = path[0] + os.path.sep
                winreg.CloseKey(dcs_path_key)
                return dcs_dir
            except FileNotFoundError:
                print("Couldn't detect DCS World installation folder")
                return ""
        except Exception as e:
            print("Couldn't detect DCS World installation folder")
            return ""
    elif is_using_dcs_steam_edition():
        return _find_steam_dcs_directory()
    else:
        print("Couldn't detect any installed DCS World version")
        return ""


def get_dcs_saved_games_directory():
    """
    Get the save game directory for DCS World
    :return: Save game directory as string
    """
    return os.path.join(os.path.expanduser("~"), "Saved Games", "DCS")


def _find_steam_directory():
    """
    Get the Steam install directory for this computer from registry
    :return Steam installation path
    """
    try:
        steam_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\\Valve\\Steam")
        path = winreg.QueryValueEx(steam_key, "SteamPath")[0]
        winreg.CloseKey(steam_key)
        return path
    except FileNotFoundError as fnfe:
        print(fnfe)
        return ""


def _get_steam_library_folders():
    """
    Get the installation directory for Steam games
    :return List of Steam library folders where games can be installed
    """
    try:
        steam_dir = _find_steam_directory()
        """        
        For reference here is what the vdf file is supposed to look like : 

        "LibraryFolders"
        {
            "TimeNextStatsReport"        "1561832478"
            "ContentStatsID"        "-158337411110787451"
            "1"        "D:\\Games\\Steam"
            "2"        "E:\\Steam"
        }
        """
        vdf_file_location = steam_dir + os.path.sep + "steamapps" + os.path.sep + "libraryfolders.vdf"
        with open(vdf_file_location) as adf_file:
            paths = [l.split("\"")[3] for l in adf_file.readlines()[1:] if ':\\\\' in l]
            return paths
    except Exception as e:
        print(e)
        return []


def _find_steam_dcs_directory():
    """
    Find the DCS install directory for DCS World Steam Edition
    :return: Install directory as string, empty string if not found
    """
    for library_folder in _get_steam_library_folders():
        folder = library_folder + os.path.sep + "steamapps" + os.path.sep + "common" + os.path.sep + "DCSWorld"
        if os.path.isdir(folder):
            return folder + os.path.sep
    return ""


if __name__ == "__main__":
    print("Using Windows : " + str(is_windows_os))
    print("Using STEAM Edition : " + str(is_using_dcs_steam_edition()))
    print("Using Standalone Edition : " + str(is_using_dcs_standalone_edition()))
    print("DCS Installation directory : " + get_dcs_install_directory())
    print("DCS saved games directory : " + get_dcs_saved_games_directory())


from typing import Dict, Any, Optional, List, Union


def loads(tablestr, _globals: Optional[Dict[str, Any]] = None):

    class Parser:
        def __init__(self, buffer: str, _globals: dict = None):
            self.buffer: str = buffer
            if _globals:
                self.variables = _globals.copy()
            else:
                self.variables = {}

            self._variable_assignment_queue: List[str] = []

            self.buflen: int = len(buffer)
            self.pos: int = 0
            self.lineno: int = 1

        def parse(self):
            self.eat_ws()

            c = self.buffer[self.pos]
            if c == '{':
                return self.object()
            elif c == '"':
                return self.string()
            elif c.isnumeric() or c == '-':
                return self.number()
            elif c == '_':
                return self.str_function()
            else:  # varname
                self.eat_ws()

                varname = self.eatvarname()
                if varname == 'false' or varname == 'true':
                    return varname == 'true'
                elif varname == 'local':
                    self.eat_ws()
                    # push names for assignment into queue
                    self.push_assigned_variable_names(self.eatvarnamelist())

                elif varname == 'return':
                    self.eat_ws()

                    name = self.eatvarname()
                    return {name: self.variables[name]}
                elif varname in self.variables:
                    # substitute value from variables
                    return self.variables[varname]
                else:
                    # shortcut syntax without `local`
                    self.push_assigned_variable_names([varname])

                self.eat_ws()
                if not self.eob() and self.buffer[self.pos] == '=':
                    while True:
                        # skip comma or whitespace
                        self.advance()

                        self.eat_ws()
                        self.assign_local_variable(self.parse())

                        if self.eob():
                            return None

                        self.eat_ws()

                        if self.eob():
                            # file ended on variable declaration
                            return None
                        elif self.char() != ',':
                            # value list ended
                            break

                    return self.parse()
                else:
                    se = SyntaxError()
                    se.text = varname + " '" + self.buffer[self.pos] + "'"
                    se.lineno = self.lineno
                    se.offset = self.pos
                    raise se

        def str_function(self) -> str:
            if self.buffer[self.pos] != '_':
                se = SyntaxError()
                se.lineno = self.lineno
                se.offset = self.pos
                se.text = "Expected character '_', got '{char}'".format(char=self.buffer[self.pos])
                raise se

            if self.advance():
                raise self.eob_exception()

            self.eat_ws()

            if self.buffer[self.pos] != '(':
                se = SyntaxError()
                se.lineno = self.lineno
                se.offset = self.pos
                se.text = "Expected character '(', got '{char}'".format(char=self.buffer[self.pos])
                raise se

            self.advance()
            self.eat_ws()

            s = self.string()

            self.eat_ws()

            if self.buffer[self.pos] != ')':
                se = SyntaxError()
                se.lineno = self.lineno
                se.offset = self.pos
                se.text = "Expected character ')', got '{char}'".format(char=self.buffer[self.pos])
                raise se

            self.pos += 1
            return s

        def string(self) -> str:
            if self.buffer[self.pos] != '"':
                se = SyntaxError()
                se.lineno = self.lineno
                se.offset = self.pos
                se.text = "Expected character '\"', got '{char}'".format(char=self.buffer[self.pos])
                raise se

            state = 0
            s = ''
            while state != 1:
                if self.advance():
                    raise self.eob_exception()

                c = self.buffer[self.pos]
                if state == 0:
                    if c == '"':
                        state = 1
                        self.pos += 1
                    elif c == '\\':
                        state = 2
                    else:
                        s += c
                elif state == 2:
                    s += c
                    state = 0
            return s

        def number(self) -> Union[int, float]:
            n = ''
            sign = 1
            if self.buffer[self.pos] == '-':
                sign = -1
                if self.advance():
                    raise self.eob_exception()

            found_exponent, found_exponent_sign, found_separator = False, False, False
            while not self.eob():
                if self.buffer[self.pos].isnumeric():
                    pass
                elif self.buffer[self.pos] == '.':
                    if not found_separator:
                        found_separator = True
                    else:
                        raise SyntaxError()
                elif self.buffer[self.pos].lower() == 'e':
                    if not found_exponent:
                        found_exponent = True
                    else:
                        raise SyntaxError()
                elif self.buffer[self.pos] == '-':
                    if not found_exponent_sign:
                        found_exponent_sign = True
                    else:
                        raise SyntaxError()
                else:
                    break

                n += self.buffer[self.pos]
                self.pos += 1

            num = float(n) * sign
            if num.is_integer():
                return int(num)
            return num

        def object(self) -> Dict[Union[int, str], Any]:
            d = {}
            if self.buffer[self.pos] != '{':
                se = SyntaxError()
                se.lineno = self.lineno
                se.offset = self.pos
                se.text = "Expected character '{', got '{char}'".format(char=self.buffer[self.pos])
                raise se

            if self.advance():
                raise self.eob_exception()

            self.eat_ws()

            inc_key = 1
            while self.buffer[self.pos] != '}':
                self.eat_ws()

                if self.buffer[self.pos] == '[':
                    # key given
                    if self.advance():
                        raise self.eob_exception()

                    self.eat_ws()
                    if self.buffer[self.pos] == '"':
                        key = self.string()
                    else:
                        key = self.number()

                    if self.eob():
                        raise self.eob_exception()

                    self.eat_ws()

                    if self.buffer[self.pos] != ']':
                        se = SyntaxError()
                        se.lineno = self.lineno
                        se.offset = self.pos
                        se.text = "Expected character ']', got '{char}'".format(char=self.buffer[self.pos])
                        raise se

                    if self.advance():
                        raise self.eob_exception()

                    self.eat_ws()

                    if self.buffer[self.pos] != '=':
                        se = SyntaxError()
                        se.lineno = self.lineno
                        se.offset = self.pos
                        se.text = "Expected character '=', got '{char}'".format(char=self.buffer[self.pos])
                        raise se

                    if self.advance():
                        raise self.eob_exception()
                else:
                    key = inc_key
                    inc_key += 1

                val = self.parse()

                self.eat_ws()

                d[key] = val
                # print(key, ':', val)

                if self.eob():
                    raise self.eob_exception()

                c = self.buffer[self.pos]
                if c == '}':
                    break
                elif c == ',':
                    if self.advance():
                        raise self.eob_exception()
                    self.eat_ws()
                else:
                    se = SyntaxError()
                    se.lineno = self.lineno
                    se.offset = self.pos
                    se.text = "Unexpected character '{char}'".format(char=self.buffer[self.pos])
                    raise se

            self.pos += 1

            return d

        def push_assigned_variable_names(self, variable_names: List[str]):
            self._variable_assignment_queue += variable_names

        def assign_local_variable(self, value):
            self.variables[self._variable_assignment_queue.pop(0)] = value

        def eatvarname(self) -> str:
            varname = ''
            while (not self.eob()) and (self.buffer[self.pos].isalnum() or self.buffer[self.pos] == '_'):
                varname += self.buffer[self.pos]
                self.pos += 1

            return varname

        def eatvarnamelist(self) -> List[str]:
            varnames = []
            while not self.eob():
                self.eat_ws()
                name = self.eatvarname()
                if len(name) > 0:
                    varnames.append(name)
                self.eat_ws()

                if self.char() == ',':
                    self.advance()
                else:
                    break

            return varnames

        def eat_comment(self):
            if (self.pos + 1 < self.buflen and
                self.buffer[self.pos] == '-' and
               self.buffer[self.pos + 1] == '-'):
                while not self.eob() and self.buffer[self.pos] != '\n':
                    self.pos += 1

        def eat_ws(self):
            """
            Advances the internal buffer until it reaches a non comment or whitespace.
            :return: None
            """
            self.eat_comment()
            while True:
                if self.pos >= self.buflen:
                    return
                c: str = self.buffer[self.pos]
                if c == '\n':
                    self.lineno += 1
                if c == '-':
                    self.eat_comment()
                    c = self.buffer[self.pos]
                if not c.isspace():
                    return

                self.pos += 1

        def eob(self) -> bool:
            """
            Checks if we are at the end of buffer.

            :return: True if end of buffer is reached, else False.
            """
            return self.pos >= self.buflen

        def eob_exception(self):
            se = SyntaxError()
            se.lineno = self.lineno
            se.offset = self.pos
            se.text = "Unexpected end of buffer"
            return se

        def char(self):
            return self.buffer[self.pos]

        def advance(self) -> bool:
            """
            Advances the internal buffer position by 1 and checks if we are at the end of buffer.
            :return: True if end of buffer is reached, else False.
            """
            self.pos += 1
            return self.eob()

    p = Parser(tablestr, _globals)
    p.parse()
    return p.variables



"""
A nav target point is special type of point handled in DCS mission editor.

Two aircraft uses these special waypoints for now :
- The JF-17 Thunder
- The F-14B Tomcat


--------------------------------------------------------------------------
JF-17 :
For the JF-17 it is possible to set up "PP" waypoint and "RP" waypoint.

PP waypoints can be set by putting a comment : PP1, PP2, PP3, PP4, or PP5
RP waypoints can be set by putting a comment : RP1, RP2, RP3, RP4, or RP5

PP waypoints are preplanned target waypoints for guided bombs.
RP waypoints are special points for the C-802AKG cruise missile.

--------------------------------------------------------------------------
F-14B Tomcat :

See http://www.heatblur.se/F-14Manual/dcs.html#f-14-waypoints-in-the-mission-editor

--------------------------------------------------------------------------

"""

from dcs import mapping


class NavTargetPoint:
    def __init__(self):
        self.position = mapping.Point(0, 0)
        self.text_comment = ""
        self.index = 0

    @classmethod
    def create_from_dict(cls, d):
        t = cls()
        t.position = mapping.Point(d["x"], d["y"])
        t.text_comment = d["text_comment"]
        t.index = d["index"]
        return t

    def dict(self):
        return {
            "x": self.position.x,
            "y": self.position.y,
            "index": self.index,
            "text_comment": self.text_comment,
        }
